// Copyright (c) 1998, 1999, 2000 Jan Luebbe. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Jan Luebbe can be emailed at jluebbe@lasnet.de

unit WFDiff;

interface

uses
  Windows, Messages, Classes, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Main, ComCtrls, Buttons;

type
  TWFDiffForm = class(TForm)
    FileNamePanel: TPanel;
    OriginalLabel: TLabel;
    ModifiedLabel: TLabel;
    OStaIdxEdit: TEdit;
    MStaIdxEdit: TEdit;
    OStaticsEdit: TEdit;
    MStaticsEdit: TEdit;
    OMapEdit: TEdit;
    MMapEdit: TEdit;
    StaIdxLabel: TLabel;
    StaticsLabel: TLabel;
    MapLabel: TLabel;
    ConfigurationPanel: TPanel;
    GenerateSelectGroupBox: TGroupBox;
    MapCheckBox: TCheckBox;
    StaticsCheckBox: TCheckBox;
    WFDiffListBox: TListBox;
    ButtonGroupBox: TGroupBox;
    GenerateButton: TButton;
    ApplyButton: TButton;
    WFDFileOpenDialog: TOpenDialog;
    WFDFileSaveDialog: TSaveDialog;
    StaDifLabel: TLabel;
    StaDifEdit: TEdit;
    StaDifiLabel: TLabel;
    StaDifiEdit: TEdit;
    StaDiflLabel: TLabel;
    StaDiflEdit: TEdit;
    MapDifLabel: TLabel;
    MapDifEdit: TEdit;
    MapDiflLabel: TLabel;
    MapDiflEdit: TEdit;
    OSIDiffCheckBox: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure MapCheckBoxClick(Sender: TObject);
    procedure StaticsCheckBoxClick(Sender: TObject);
    Procedure FilenameChange(Sender: TObject);
    procedure GenerateButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure OSIDiffCheckBoxClick(Sender: TObject);
  private
    // UI
    Procedure UpdateEnabled;
   // procedure UpdateTip;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  WFDiffForm: TWFDiffForm; // = nil???

type
  TWFDStaticsIndexRecord = packed record
    StaticsStart: longword;
    StaticsLength: longword;
  end;
  TWFDMapIndexRecord = packed record
    MapStart: longword;
  end;
  TWFDInfo = packed record
    Version: longint; // must be = const WFDVersion
    StaticsIndexStart: longword; // no statics if $FFFFFFFF
    OrigStaIdxCRC: longword;
    OrigStaticsSize: longword;
    ModStaIdxCRC: longword;
    ModStaticsSize: longword;
    MapIndexStart: longword; // no map if $FFFFFFFF
    ModMapCRC: longword;
    Reserved: array[1..992] of byte;
  end;
  TWFDStaticsIndex = Array of Array of TWFDStaticsIndexRecord;
  TWFDMapIndex = Array of Array of TWFDMapIndexRecord;
  TWFDFile = file of byte;
  TWFStaDifFile = file of byte;
  TWFStaDifiFile = file of byte;
  TWFStaDiflFile = file of byte;
  TWFMapDifFile = file of byte;
  TWFMapDiflFile = file of byte;

const
  WFDVersion = 2;

// StaticIndex CRC
function CRCStaticIndex(const StaticIndex: TStaticIndex):longword;

// Map?.mul CRC
function CRCMap(const FileName: string; const SysProgress: TSysProgress):longword;

// Compare
function StaticPointerEqual(const PointerA, PointerB: TStaticPointer): boolean;
function StaticBlocksEqual(const BlockA, BlockB: TStaticBlock): boolean;
function MapBlocksEqual(const BlockA, BlockB: TMapBlock): boolean;

// WFDiff file
procedure AppendStaticsDiff(var WFDFile: TWFDFile; var IndexRecord: TWFDStaticsIndexRecord; const StaticBlock: TStaticBlock);
procedure GetStaticsDiff(var WFDFile: TWFDFile; const IndexRecord: TWFDStaticsIndexRecord; var StaticBlock: TStaticBlock);
procedure AppendMapDiff(var WFDFile: TWFDFile; var IndexRecord: TWFDMapIndexRecord; const MapBlock: TMapBlock);
procedure GetMapDiff(var WFDFile: TWFDFile; const IndexRecord: TWFDMapIndexRecord; var MapBlock: TMapBlock);

// Global
procedure GenerateOSIDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                          const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                          const SysProgress: TSysProgress);
procedure GenerateDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                       const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                       const WFDiffFileName: string;
                       const SysProgress: TSysProgress);
procedure ApplyDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                    const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                    const WFDiffFileName: string;
                    const SysProgress: TSysProgress);

// Registry
Procedure LoadFileNames(Var MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
  StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName: String);
Procedure SaveFileNames(Const MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
  StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName: String);

implementation

{$R *.DFM}

uses SysUtils, Graphics, Registry, Tools, Progress;

// StaticIndex CRC
function CRCStaticIndex(const StaticIndex: TStaticIndex):longword;
var
  X, Y: word;
begin
  Result := 0;
  for X := 0 to MainForm.MapWidth-1 do
  begin
    for Y := 0 to MainForm.MapHeight-1 do
    begin
      Result := Result xor StaticIndex[X, Y].Start;
      Result := Result xor StaticIndex[X, Y].Length;
      Result := Result xor StaticIndex[X, Y].Unknown;
    end;
    Application.ProcessMessages;
  end;
end;

// Map?.mul CRC
function CRCMap(const FileName: string; const SysProgress: TSysProgress):longword;
type
  TCRCMapBlock = packed array[0..48] of longword;
var
  i, j: longint;
  TempFile: file of byte;
  CRCMapBlock: TCRCMapBlock;
begin
  Assert(SizeOf(TCRCMapBlock)=SizeOf(TMapBlock), 'SizeOf() error while calculating CRC of map?.mul');
  SysProgress(0);
  Result := 0;
  AssignFile(TempFile, FileName);
  Reset(TempFile);
  for i := 1 to MainForm.MapWidth*MainForm.MapHeight do
  begin
    BlockRead(TempFile, CRCMapBlock, SizeOf(TCRCMapBlock));
    for j := 0 to 48 do
    begin
      Result := Result xor CRCMapBlock[j];
    end;
    SysProgress(i*100 div (MainForm.MapWidth*MainForm.MapHeight));
    Application.ProcessMessages;
  end;
  CloseFile(TempFile);
  SysProgress(100);
end;

// Compare
function StaticPointerEqual(const PointerA, PointerB: TStaticPointer): boolean;
begin
  if (PointerA.Start = PointerB.Start) and
     (PointerA.Length = PointerB.Length) then Result := True // nur wenn wirklich klar ist,
  else Result := False;                                      // dass die pointer auf die gleichen daten
                                                             // daten zeigen, true zurueckgeben!
end;

function StaticBlocksEqual(const BlockA, BlockB: TStaticBlock): boolean;
begin
  Result := False;
  if not (BlockA.Count = BlockB.Count) then Exit;
  Result := CompareMem(Addr(BlockA), Addr(BlockB), SizeOf(TStaticBlock));
end;

function MapBlocksEqual(const BlockA, BlockB: TMapBlock): boolean;
//var
  //AUnknown, BUnknown: integer;
begin
  //AUnknown := BlockA.Unknown;
  //BUnknown := BlockB.Unknown;
  //BlockA.Unknown := 0;
  //BlockB.Unknown := 0;
  Result := CompareMem(Addr(BlockA), Addr(BlockB), SizeOf(TMapBlock));
  //BlockA.Unknown := AUnknown;
  //BlockB.Unknown := BUnknown;
end;

// WFDiff file
procedure AppendStaticsDiff(var WFDFile: TWFDFile; var IndexRecord: TWFDStaticsIndexRecord;  const StaticBlock: TStaticBlock);
var
  i: word;
begin
  Seek(WFDFile, FileSize(WFDFile));
  IndexRecord.StaticsStart := FilePos(WFDFile);
  IndexRecord.StaticsLength := StaticBlock.Count*7;

  for i := 1 to StaticBlock.Count do
    BlockWrite(WFDFile, StaticBlock.Rec[i], SizeOf(TStaticRec));
end;

procedure GetStaticsDiff(var WFDFile: TWFDFile; const IndexRecord: TWFDStaticsIndexRecord; var StaticBlock: TStaticBlock);
var
  i: word;
begin
  Seek(WFDFile, IndexRecord.StaticsStart);
  StaticBlock.Count := IndexRecord.StaticsLength div 7;

  for i := 1 to StaticBlock.Count do
    BlockRead(WFDFile, StaticBlock.Rec[i], SizeOf(TStaticRec));
end;

procedure AppendMapDiff(var WFDFile: TWFDFile; var IndexRecord: TWFDMapIndexRecord; const MapBlock: TMapBlock);
begin
  Seek(WFDFile, FileSize(WFDFile));
  IndexRecord.MapStart := FilePos(WFDFile);

  BlockWrite(WFDFile, MapBlock, SizeOf(TMapBlock));
end;

procedure GetMapDiff(var WFDFile: TWFDFile; const IndexRecord: TWFDMapIndexRecord; var MapBlock: TMapBlock);
begin
  Seek(WFDFile, IndexRecord.MapStart);

  BlockRead(WFDFile, MapBlock, SizeOf(TMapBlock));
end;

// Global
// 2004/05/06 - Birdy
procedure GenerateOSIDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                          const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                          const SysProgress: TSysProgress);
var
  X, Y: word;
  OStaticIndex, MStaticIndex: TStaticIndex;
  WFDStaticsIndex: TWFDStaticsIndex;
  WFDMapIndex: TWFDMapIndex;
  pOStaticBlock, pMStaticBlock: ^TStaticBlock;
  pOMapBlock, pMMapBlock: ^TMapBlock;
  WFStaDifFile: MulFileType;
  WFStaDifiFile: TWFStaDifiFile;
  WFStaDiflFile: TWFStaDiflFile;
  MapDifFile: file of byte;
  MapDiflFile: file of byte;
  StaDifFileName: String;
  StaDifiFileName: String;
  TempFile: file of byte;
  //OriginalCRC: integer;
 // TheIdxCount: longword;
 // TheDifCount: longword;
  TheMapBlock: longword;
  MapDifFileName: String;
  MapDiflFileName: String;
begin
  TheMapBlock:=0;

  SetLength(OStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(MStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDStaticsIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDMapIndex, MainForm.MapWidth, MainForm.MapHeight);
  New(pOStaticBlock);
  New(pMStaticBlock);
  New(pOMapBlock);
  New(pMMapBlock);
  ProgressForm.Show;

  try
    begin // clear memory
      ZeroMemory(OStaticIndex, SizeOf(OStaticIndex));
      ZeroMemory(MStaticIndex, SizeOf(MStaticIndex));
      FillMemory(WFDStaticsIndex, SizeOf(WFDStaticsIndex), $FF);
      FillMemory(WFDMapIndex, SizeOf(WFDMapIndex), $FF);
    end;

    StaDifFileName := WFDiffForm.StaDifEdit.Text;
    StaDifiFileName := WFDiffForm.StaDifiEdit.Text;
    MapDifFileName := WFDiffForm.MapDifEdit.Text;
    MapDiflFileName := WFDiffForm.MapDiflEdit.Text;

    if WFDiffForm.StaticsCheckBox.Checked then
    begin // Statics0.mul
      AssignFile(WFStaDifiFile, StaDifiFileName);
      Rewrite(WFStaDifiFile);

      AssignFile(WFStaDifFile, StaDifFileName);
      Rewrite(WFStaDifFile);
//      CloseFile(WFStaDifFile);

      AssignFile(WFStaDiflFile, WFDiffForm.StaDiflEdit.Text);
      Rewrite(WFStaDiflFile);

      ProgressForm.SetText('Generating Diff (Statics)');

      GetStaticIndex(OStaIdxFileName, OStaticIndex, SysProgress);
      GetStaticIndex(MStaIdxFileName, MStaticIndex, SysProgress);

      AssignFile(TempFile, OStaticsFileName);
      Reset(TempFile);
      CloseFile(TempFile);

      AssignFile(TempFile, MStaticsFileName);
      Reset(TempFile);
      CloseFile(TempFile);

      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          TheMapBlock := (X * MainForm.MapHeight) + Y;
          if ((MStaticIndex[X, Y].Start = $FFFFFFFF) and (OStaticIndex[X, Y].Start = $FFFFFFFF)) then
              continue;

          if ((MStaticIndex[X, Y].Start = $FFFFFFFF) or (OStaticIndex[X, Y].Start = $FFFFFFFF)) then
            begin
              SysLog('Changing Static Block @ ('+IntToStr(X*8)+','+IntToStr(Y*8)+') ['+IntToStr(TheMapBlock)+']');
              BlockWrite(WFStaDiflFile, TheMapBlock, 4);
              if not (MStaticIndex[X, Y].Start = $FFFFFFFF) then
                begin
                  GetStaticBlock(MStaticsFileName, MStaticIndex[X, Y], PMStaticBlock^);
                  PutStaticBlockToFile(WFStaDifFile, MStaticIndex[X, Y], PMStaticBlock^);
//                  PutStaticBlock(StaDifFileName, MStaticIndex[X, Y], PMStaticBlock^);
                end
              else
                begin
                  MStaticIndex[X, Y].Length := $FFFFFFFF;
                end;
              BlockWrite(WFStaDifiFile, MStaticIndex[X, Y], SizeOf(TStaticPointer));
            end
          else
            begin
              GetStaticBlock(OStaticsFileName, OStaticIndex[X, Y], pOStaticBlock^);
              GetStaticBlock(MStaticsFileName, MStaticIndex[X, Y], pMStaticBlock^);
              if not StaticBlocksEqual(POStaticBlock^, PMStaticBlock^) then // they are really different
              begin
                SysLog('Changing Static Block @ ('+IntToStr(X*8)+','+IntToStr(Y*8)+') ['+IntToStr(TheMapBlock)+']');
                BlockWrite(WFStaDiflFile, TheMapBlock, 4);
                PutStaticBlockToFile(WFStaDifFile, MStaticIndex[X, Y], PMStaticBlock^);
//                PutStaticBlock(StaDifFileName, MStaticIndex[X, Y], PMStaticBlock^);
                BlockWrite(WFStaDifiFile, MStaticIndex[X, Y], SizeOf(TStaticPointer));
              end;
            end;

        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      CloseFile(WFStaDifFile);
      CloseFile(WFStaDifiFile);
      CloseFile(WFStaDiflFile);
    end;

    if WFDiffForm.MapCheckBox.Checked then
    begin // Map0.mul
      AssignFile(MapDifFile, MapDifFileName);
      Rewrite(MapDifFile);

      AssignFile(MapDiflFile, MapDiflFileName);
      Rewrite(MapDiflFile);

      ProgressForm.SetText('Generating Diff (Map)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          TheMapBlock := (X * MainForm.MapHeight) + Y;
          GetMapBlock(OMapFileName, X, Y, pOMapBlock^);
          GetMapBlock(MMapFileName, X, Y, pMMapBlock^);
          if not MapBlocksEqual(pOMapBlock^, pMMapBlock^) then
          begin
            SysLog('Changing Map Block @ ('+IntToStr(X*8)+','+IntToStr(Y*8)+') ['+IntToStr(TheMapBlock)+']');
            BlockWrite(MapDiflFile, TheMapBlock, 4);
            BlockWrite(MapDifFile, pMMapBlock^, SizeOf(TMapBlock));
          end;
        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      CloseFile(MapDifFile);
      CloseFile(MapDiflFile);
    end;


  finally
    ProgressForm.Hide;
    Dispose(pOStaticBlock);
    Dispose(pMStaticBlock);
    Dispose(pOMapBlock);
    Dispose(pMMapBlock);
  end;
end;

procedure GenerateDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                       const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                       const WFDiffFileName: string;
                       const SysProgress: TSysProgress);
var
  X, Y: word;
  WFDInfo: TWFDInfo;
  OStaticIndex, MStaticIndex: TStaticIndex;
  WFDStaticsIndex: TWFDStaticsIndex;
  WFDMapIndex: TWFDMapIndex;
  pOStaticBlock, pMStaticBlock: ^TStaticBlock;
  pOMapBlock, pMMapBlock: ^TMapBlock;
  WFDFile: TWFDFile;
  TempFile: file of byte;
  //OriginalCRC: integer;
begin
  SetLength(OStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(MStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDStaticsIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDMapIndex, MainForm.MapWidth, MainForm.MapHeight);
  New(pOStaticBlock);
  New(pMStaticBlock);
  New(pOMapBlock);
  New(pMMapBlock);
  ProgressForm.Show;

  try
    begin // clear memory
      ZeroMemory(Addr(WFDInfo), SizeOf(TWFDInfo));
      ZeroMemory(OStaticIndex, SizeOf(OStaticIndex));
      ZeroMemory(MStaticIndex, SizeOf(MStaticIndex));
      FillMemory(WFDStaticsIndex, SizeOf(WFDStaticsIndex), $FF);
      FillMemory(WFDMapIndex, SizeOf(WFDMapIndex), $FF);
    end;
    AssignFile(WFDFile, WFDiffFileName);
    Rewrite(WFDFile);

    with WFDInfo do
    begin
      Version := WFDVersion;
      StaticsIndexStart := $FFFFFFFF;
      MapIndexStart := $FFFFFFFF;
    end;

    BlockWrite(WFDFile, WFDInfo, SizeOf(WFDInfo)); // Write 1024 byte header

    if WFDiffForm.StaticsCheckBox.Checked then
    begin // Statics0.mul
      ProgressForm.SetText('Generating Diff (Statics)');

      GetStaticIndex(OStaIdxFileName, OStaticIndex, SysProgress);
      GetStaticIndex(MStaIdxFileName, MStaticIndex, SysProgress);

      WFDInfo.OrigStaIdxCRC := CRCStaticIndex(OStaticIndex);
      WFDInfo.ModStaIdxCRC := CRCStaticIndex(MStaticIndex);

      AssignFile(TempFile, OStaticsFileName);
      Reset(TempFile);
      WFDInfo.OrigStaticsSize := FileSize(TempFile);
      CloseFile(TempFile);

      AssignFile(TempFile, MStaticsFileName);
      Reset(TempFile);
      WFDInfo.ModStaticsSize := FileSize(TempFile);
      CloseFile(TempFile);

      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          {if (OStaticIndex[X, Y].Start = $FFFFFFFF)
            and (MStaticIndex[X, Y].Start = $FFFFFFFF) then
          begin // no block in both original and modified version
            WFDStaticsIndex[X, Y].StaticsStart := $FFFFFFFF;
            WFDStaticsIndex[X, Y].StaticsLength := $FFFFFFFF;
            Continue;
          end;} // covered by next one
          if (MStaticIndex[X, Y].Start = $FFFFFFFF) then
          begin // block removed
            WFDStaticsIndex[X, Y].StaticsStart := $FFFFFFFF;
            WFDStaticsIndex[X, Y].StaticsLength := 0;
            Continue;
          end;
          if StaticPointerEqual(OStaticIndex[X, Y], MStaticIndex[X, Y]) then
          begin // blocks must be equal
            WFDStaticsIndex[X, Y].StaticsStart := $FFFFFFFF;
            WFDStaticsIndex[X, Y].StaticsLength := $FFFFFFFF;
          end
          else
          begin // blocks may be different
            GetStaticBlock(OStaticsFileName, OStaticIndex[X, Y], POStaticBlock^);
            GetStaticBlock(MStaticsFileName, MStaticIndex[X, Y], PMStaticBlock^);
            if not StaticBlocksEqual(POStaticBlock^, PMStaticBlock^) then // they are really different
            begin
              AppendStaticsDiff(WFDFile, WFDStaticsIndex[X, Y], PMStaticBlock^);
            end;
          end;
        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      // Write WDFInfo.StaticsIndexStart and StaticsIndex
      WFDInfo.StaticsIndexStart := FilePos(WFDFile);
      ProgressForm.SetText('Generating Diff (Statics index)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
          BlockWrite(WFDFile, WFDStaticsIndex[X, Y], SizeOf(TWFDStaticsIndexRecord));
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;
    end;

    if WFDiffForm.MapCheckBox.Checked then
    begin // Map0.mul
      WFDInfo.ModMapCRC := CRCMap(MMapFileName, SysProgress);
      ProgressForm.SetText('Generating Diff (Map)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          GetMapBlock(OMapFileName, X, Y, pOMapBlock^);
          GetMapBlock(MMapFileName, X, Y, pMMapBlock^);
          if not MapBlocksEqual(pOMapBlock^, pMMapBlock^) then
          begin
            AppendMapDiff(WFDFile, WFDMapIndex[X, Y], pMMapBlock^);
          end;
        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      // Write WDFInfo.MapIndexStart and MapIndex
      WFDInfo.MapIndexStart := FilePos(WFDFile);
      ProgressForm.SetText('Generating Diff (Map index)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
          BlockWrite(WFDFile, WFDMapIndex[X, Y], SizeOf(TWFDMapIndexRecord));
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;
    end;

    Seek(WFDFile, 0);

    BlockWrite(WFDFile, WFDInfo, SizeOf(WFDInfo)); // Write 1024 byte header

    Application.MessageBox(PChar('The original statics file has a CRC value of '+IntToSpacedHex(WFDInfo.OrigStaIdxCRC, 4)+'.') , 'CRC value', MB_OK)

  finally
    CloseFile(WFDFile);
    ProgressForm.Hide;
    Dispose(pOStaticBlock);
    Dispose(pMStaticBlock);
    Dispose(pOMapBlock);
    Dispose(pMMapBlock);
  end;
end;

procedure ApplyDiff(const OStaIdxFileName, OStaticsFileName, OMapFileName: string;
                    const MStaIdxFileName, MStaticsFileName, MMapFileName: string;
                    const WFDiffFileName: string;
                    const SysProgress: TSysProgress);
var
  X, Y: word;
  WFDInfo: TWFDInfo;
  OStaticIndex, MStaticIndex: TStaticIndex;
  WFDStaticsIndex: TWFDStaticsIndex;
  WFDMapIndex: TWFDMapIndex;
  pStaticBlock: ^TStaticBlock;
  pMapBlock: ^TMapBlock;
  WFDFile: TWFDFile;
  OriginalStaticsCRC: longword;
  TempFile: file of byte;
begin
  SetLength(OStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(MStaticIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDStaticsIndex, MainForm.MapWidth, MainForm.MapHeight);
  SetLength(WFDMapIndex, MainForm.MapWidth, MainForm.MapHeight);
  New(pStaticBlock);
  New(pMapBlock);
  ProgressForm.Show;

  try
    begin // clear memory
      ZeroMemory(OStaticIndex, SizeOf(OStaticIndex));
      FillMemory(MStaticIndex, SizeOf(MStaticIndex), $FF);
      ZeroMemory(WFDStaticsIndex, SizeOf(WFDStaticsIndex));
      ZeroMemory(WFDMapIndex, SizeOf(WFDMapIndex));
    end;
    AssignFile(WFDFile, WFDiffFileName);
    Reset(WFDFile);

    BlockRead(WFDFile, WFDInfo, SizeOf(TWFDInfo));

    if not (WFDInfo.StaticsIndexStart = $FFFFFFFF) then // Statics
    begin
      GetStaticIndex(OStaIdxFileName, OStaticIndex, SysProgress);

      OriginalStaticsCRC := CRCStaticIndex(OStaticIndex);

      if not (OriginalStaticsCRC = WFDInfo.OrigStaIdxCRC) then
      begin
        Application.MessageBox(PChar('The CRC value of your orginal statics file ('+IntToSpacedHex(OriginalStaticsCRC, 4)+') does not match the one used to generate this WFD file ('+IntToSpacedHex(WFDInfo.OrigStaIdxCRC, 4)+'). To apply this WFD you need to get the correct original files!') , 'CRC error!', MB_OK);
        Exit;
      end;

      AssignFile(TempFile, MStaticsFileName); // Statics leeren
      Rewrite(TempFile);
      CloseFile(TempFile);

      ProgressForm.SetText('Applying Diff (Reading statics index)');
      Seek(WFDFile, WFDInfo.StaticsIndexStart);
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
          BlockRead(WFDFile, WFDStaticsIndex[X, Y], SizeOf(TWFDStaticsIndexRecord));
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      ProgressForm.SetText('Applying Diff (Creating statics)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          if (WFDStaticsIndex[X, Y].StaticsStart = $FFFFFFFF)
            and (WFDStaticsIndex[X, Y].StaticsLength = $FFFFFFFF) then
          begin // use block from original version
            if (OStaticIndex[X, Y].Length = 0) then
            begin
              MStaticIndex[X, Y].Start := $FFFFFFFF;
              MStaticIndex[X, Y].Length := $FFFFFFFF;
              MStaticIndex[X, Y].Unknown := OStaticIndex[X, Y].Unknown; // eventuell in WFD speichern?
              Continue;
            end;
            GetStaticBlock(OStaticsFileName, OStaticIndex[X, Y], PStaticBlock^);
            PutStaticBlock(MStaticsFileName, MStaticIndex[X, Y], PStaticBlock^);
            MStaticIndex[X, Y].Unknown := OStaticIndex[X, Y].Unknown; // eventuell in WFD speichern?
            Continue;
          end;
          if (WFDStaticsIndex[X, Y].StaticsStart = $FFFFFFFF)
            and (WFDStaticsIndex[X, Y].StaticsLength = 0) then
          begin // write empty block
            MStaticIndex[X, Y].Start := $FFFFFFFF;
            MStaticIndex[X, Y].Length := $FFFFFFFF;
            MStaticIndex[X, Y].Unknown := OStaticIndex[X, Y].Unknown; // eventuell in WFD speichern?
            Continue;
          end;
          begin // write block from WFD
            GetStaticsDiff(WFDFile, WFDStaticsIndex[X, Y], PStaticBlock^);
            PutStaticBlock(MStaticsFileName, MStaticIndex[X, Y], PStaticBlock^);
          end;
        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      PutStaticIndex(MStaIdxFileName, MStaticIndex, SysProgress);
    end;

    if not (WFDInfo.MapIndexStart = $FFFFFFFF) then // Map
    begin
      Seek(WFDFile, WFDInfo.MapIndexStart);
      ProgressForm.SetText('Applying Diff (Reading map index)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
          BlockRead(WFDFile, WFDMapIndex[X, Y], SizeOf(TWFDMapIndexRecord));
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;

      ProgressForm.SetText('Applying Diff (Creating map)');
      for X := 0 to MainForm.MapWidth-1 do
      begin
        for Y := 0 to MainForm.MapHeight-1 do
        begin
          if (WFDMapIndex[X, Y].MapStart = $FFFFFFFF) then
          begin // use block from original version
            GetMapBlock(OMapFileName, X, Y, pMapBlock^);
            SetMapBlock(MMapFileName, X, Y, pMapBlock^);
          end
          else
          begin // write block from WFD
            GetMapDiff(WFDFile, WFDMapIndex[X, Y], pMapBlock^);
            SetMapBlock(MMapFileName, X, Y, pMapBlock^);
          end;
        end;
        ProgressForm.SetPercent(X*100 div MainForm.MapWidth);
        Application.ProcessMessages;
      end;
    end;

  finally
    CloseFile(WFDFile);
    ProgressForm.Hide;
    Dispose(PStaticBlock);
    Dispose(pMapBlock);
  end;
end;

// Registry
// 2004/05/25 - Shinigami
Procedure LoadFileNames(Var MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
  StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName: String);
Var Registry: TRegistry;
Begin
  Registry:=TRegistry.Create;
  Registry.OpenKey('\Software\Ridcully\WorldForge', False);

  Case MainForm.MapComboBox.ItemIndex of
    Map0_Britannia: Begin
                      MapFileName     :=Registry.ReadString('Map0.mul');
                      StaIdxFileName  :=Registry.ReadString('StaIdx0.mul');
                      StaticsFileName :=Registry.ReadString('Statics0.mul');
                      MapDifFileName  :=Registry.ReadString('MapDif0.mul');
                      MapDiflFileName :=Registry.ReadString('MapDifl0.mul');
                      StaDifFileName  :=Registry.ReadString('StaDif0.mul');
                      StaDifiFileName :=Registry.ReadString('StaDifi0.mul');
                      StaDiflFileName :=Registry.ReadString('StaDifl0.mul');
                      OMapFileName    :=Registry.ReadString('O-Map0.mul');
                      OStaIdxFileName :=Registry.ReadString('O-StaIdx0.mul');
                      OStaticsFileName:=Registry.ReadString('O-Statics0.mul');
                    End;
    Map1_Britannia: Begin
                      MapFileName     :=Registry.ReadString('Map0.mul');
                      StaIdxFileName  :=Registry.ReadString('StaIdx0.mul');
                      StaticsFileName :=Registry.ReadString('Statics0.mul');
                      MapDifFileName  :=Registry.ReadString('MapDif1.mul');
                      MapDiflFileName :=Registry.ReadString('MapDifl1.mul');
                      StaDifFileName  :=Registry.ReadString('StaDif1.mul');
                      StaDifiFileName :=Registry.ReadString('StaDifi1.mul');
                      StaDiflFileName :=Registry.ReadString('StaDifl1.mul');
                      OMapFileName    :=Registry.ReadString('O-Map0.mul');
                      OStaIdxFileName :=Registry.ReadString('O-StaIdx0.mul');
                      OStaticsFileName:=Registry.ReadString('O-Statics0.mul');
                    End;
    Map2_Illshenar: Begin
                      MapFileName     :=Registry.ReadString('Map2.mul');
                      StaIdxFileName  :=Registry.ReadString('StaIdx2.mul');
                      StaticsFileName :=Registry.ReadString('Statics2.mul');
                      MapDifFileName  :=Registry.ReadString('MapDif2.mul');
                      MapDiflFileName :=Registry.ReadString('MapDifl2.mul');
                      StaDifFileName  :=Registry.ReadString('StaDif2.mul');
                      StaDifiFileName :=Registry.ReadString('StaDifi2.mul');
                      StaDiflFileName :=Registry.ReadString('StaDifl2.mul');
                      OMapFileName    :=Registry.ReadString('O-Map2.mul');
                      OStaIdxFileName :=Registry.ReadString('O-StaIdx2.mul');
                      OStaticsFileName:=Registry.ReadString('O-Statics2.mul');
                    End;
    Map3_Malas:     Begin
                      MapFileName     :=Registry.ReadString('Map3.mul');
                      StaIdxFileName  :=Registry.ReadString('StaIdx3.mul');
                      StaticsFileName :=Registry.ReadString('Statics3.mul');
                      MapDifFileName  :=Registry.ReadString('MapDif3.mul');    // gibs noch nicht
                      MapDiflFileName :=Registry.ReadString('MapDifl3.mul');
                      StaDifFileName  :=Registry.ReadString('StaDif3.mul');
                      StaDifiFileName :=Registry.ReadString('StaDifi3.mul');
                      StaDiflFileName :=Registry.ReadString('StaDifl3.mul');
                      OMapFileName    :=Registry.ReadString('O-Map3.mul');
                      OStaIdxFileName :=Registry.ReadString('O-StaIdx3.mul');
                      OStaticsFileName:=Registry.ReadString('O-Statics3.mul');
                    End;
    Map4_Tokuno:    Begin
                      MapFileName     :=Registry.ReadString('Map4.mul');
                      StaIdxFileName  :=Registry.ReadString('StaIdx4.mul');
                      StaticsFileName :=Registry.ReadString('Statics4.mul');
                      MapDifFileName  :=Registry.ReadString('MapDif4.mul');    // gibs noch nicht
                      MapDiflFileName :=Registry.ReadString('MapDifl4.mul');
                      StaDifFileName  :=Registry.ReadString('StaDif4.mul');
                      StaDifiFileName :=Registry.ReadString('StaDifi4.mul');
                      StaDiflFileName :=Registry.ReadString('StaDifl4.mul');
                      OMapFileName    :=Registry.ReadString('O-Map3.mul');
                      OStaIdxFileName :=Registry.ReadString('O-StaIdx3.mul');
                      OStaticsFileName:=Registry.ReadString('O-Statics3.mul');
                    End;
  End;

  Registry.CloseKey;
  Registry.Free;
End;

// 2004/05/25 - Shinigami
Procedure SaveFileNames(Const MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
  StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName: String);
Var Registry: TRegistry;
Begin
  Registry:=TRegistry.Create;
  Registry.OpenKey('\Software\Ridcully\WorldForge', True);

  Case MainForm.MapComboBox.ItemIndex of
    Map0_Britannia: Begin
                      Registry.WriteString('MapDif0.mul',    MapDifFileName);
                      Registry.WriteString('MapDifl0.mul',   MapDiflFileName);
                      Registry.WriteString('StaDif0.mul',    StaDifFileName);
                      Registry.WriteString('StaDifi0.mul',   StaDifiFileName);
                      Registry.WriteString('StaDifl0.mul',   StaDiflFileName);
                      Registry.WriteString('O-Map0.mul',     OMapFileName);
                      Registry.WriteString('O-StaIdx0.mul',  OStaIdxFileName);
                      Registry.WriteString('O-Statics0.mul', OStaticsFileName);
                    End;
    Map1_Britannia: Begin
                      Registry.WriteString('MapDif1.mul',    MapDifFileName);
                      Registry.WriteString('MapDifl1.mul',   MapDiflFileName);
                      Registry.WriteString('StaDif1.mul',    StaDifFileName);
                      Registry.WriteString('StaDifi1.mul',   StaDifiFileName);
                      Registry.WriteString('StaDifl1.mul',   StaDiflFileName);
                      Registry.WriteString('O-Map0.mul',     OMapFileName);
                      Registry.WriteString('O-StaIdx0.mul',  OStaIdxFileName);
                      Registry.WriteString('O-Statics0.mul', OStaticsFileName);
                    End;
    Map2_Illshenar: Begin
                      Registry.WriteString('MapDif2.mul',    MapDifFileName);
                      Registry.WriteString('MapDifl2.mul',   MapDiflFileName);
                      Registry.WriteString('StaDif2.mul',    StaDifFileName);
                      Registry.WriteString('StaDifi2.mul',   StaDifiFileName);
                      Registry.WriteString('StaDifl2.mul',   StaDiflFileName);
                      Registry.WriteString('O-Map2.mul',     OMapFileName);
                      Registry.WriteString('O-StaIdx2.mul',  OStaIdxFileName);
                      Registry.WriteString('O-Statics2.mul', OStaticsFileName);
                    End;
    Map3_Malas:     Begin
                      Registry.WriteString('MapDif3.mul',    MapDifFileName);   // gibs noch nicht
                      Registry.WriteString('MapDifl3.mul',   MapDiflFileName);
                      Registry.WriteString('StaDif3.mul',    StaDifFileName);
                      Registry.WriteString('StaDifi3.mul',   StaDifiFileName);
                      Registry.WriteString('StaDifl3.mul',   StaDiflFileName);
                      Registry.WriteString('O-Map3.mul',     OMapFileName);
                      Registry.WriteString('O-StaIdx3.mul',  OStaIdxFileName);
                      Registry.WriteString('O-Statics3.mul', OStaticsFileName);
                    End;
    Map4_Tokuno:    Begin
                      Registry.WriteString('MapDif4.mul',    MapDifFileName);   // gibs noch nicht
                      Registry.WriteString('MapDifl4.mul',   MapDiflFileName);
                      Registry.WriteString('StaDif4.mul',    StaDifFileName);
                      Registry.WriteString('StaDifi4.mul',   StaDifiFileName);
                      Registry.WriteString('StaDifl4.mul',   StaDiflFileName);
                      Registry.WriteString('O-Map4.mul',     OMapFileName);
                      Registry.WriteString('O-StaIdx4.mul',  OStaIdxFileName);
                      Registry.WriteString('O-Statics4.mul', OStaticsFileName);
                    End;
  End;

  Registry.CloseKey;
  Registry.Free;
End;

// UI
// 2004/05/26 - Shinigami
Procedure TWFDiffForm.UpdateEnabled;
Begin
  If LowerCase(OStaIdxEdit.Text)=LowerCase(MStaIdxEdit.Text) Then Begin
    OStaIdxEdit.Color:=clRed;MStaIdxEdit.Color:=clRed;
  End Else Begin
    If FileExists(OStaIdxEdit.Text) Then OStaIdxEdit.Color:=clGreen Else OStaIdxEdit.Color:=clRed;
    If FileExists(MStaIdxEdit.Text) Then MStaIdxEdit.Color:=clGreen Else MStaIdxEdit.Color:=clRed;
  End;
  If LowerCase(OStaticsEdit.Text)=LowerCase(MStaticsEdit.Text) Then Begin
    OStaticsEdit.Color:=clRed;MStaticsEdit.Color:=clRed;
  End Else Begin
    If FileExists(OStaticsEdit.Text) Then OStaticsEdit.Color:=clGreen Else OStaticsEdit.Color:=clRed;
    If FileExists(MStaticsEdit.Text) Then MStaticsEdit.Color:=clGreen Else MStaticsEdit.Color:=clRed;
  End;
  If LowerCase(OMapEdit.Text)=LowerCase(MMapEdit.Text) Then Begin
    OMapEdit.Color:=clRed;MMapEdit.Color:=clRed;
  End Else Begin
    If FileExists(OMapEdit.Text) Then OMapEdit.Color:=clGreen Else OMapEdit.Color:=clRed;
    If FileExists(MMapEdit.Text) Then MMapEdit.Color:=clGreen Else MMapEdit.Color:=clRed;
  End;

  If (OStaIdxEdit.Color=clGreen) And (OStaticsEdit.Color=clGreen) And (OMapEdit.Color=clGreen) And
     (MapDifEdit.Color=clGreen) And (MapDiflEdit.Color=clGreen) And
     (StaDifEdit.Color=clGreen) And (StaDifiEdit.Color=clGreen) And (StaDiflEdit.Color=clGreen) And
     (MStaIdxEdit.Color=clGreen) And (MStaticsEdit.Color=clGreen) And (MMapEdit.Color=clGreen) Then Begin

    SaveFileNames(MMapEdit.Text, MStaIdxEdit.Text, MStaticsEdit.Text, MapDifEdit.Text, MapDiflEdit.Text,
      StaDifEdit.Text, StaDifiEdit.Text, StaDiflEdit.Text, OMapEdit.Text, OStaIdxEdit.Text, OStaticsEdit.Text);

    StaticsCheckBox.Enabled := True;
    MapCheckBox.Enabled := True;
    GenerateButton.Enabled := True;
    ApplyButton.Enabled := Not OSIDiffCheckBox.Checked;

    OStaIdxEdit.Enabled := True;
    OStaticsEdit.Enabled := True;
    OMapEdit.Enabled := True;
    MMapEdit.Enabled := True; 

    If Mode = Simple Then Begin
      StaticsCheckBox.Enabled := False;
      MapCheckBox.Enabled := False;
      GenerateButton.Enabled := False;
    End;
  End;
End;

(* procedure TWFDiffForm.UpdateTip;
//var
  //NextStep: TStep;
begin
    {if not (Mode = Simple) then Exit;
    if (StaIdxEdit.Color = clRed) then
      NextStep := StaIdxName
    else if (StaticsEdit.Color = clRed) then
      NextStep := StaticsName
    else if WSCEdit.Color = clRed then
      NextStep := WSCName
    else if not StaticIndexLoaded then
      NextStep := GetIndex
    else if not PatchFinished then
      NextStep := FreezeNow
    else
      NextStep := Finished;
    MainListBox.Items.Clear;
    case NextStep of
      StaIdxName:
      begin
        MainListBox.Items.Add('Please enter the path of your Staidx0.mul file in');
        MainListBox.Items.Add('the left box. It will turn green when WorldForge');
        MainListBox.Items.Add('found the file.');
      end;
      StaticsName:
      begin
        MainListBox.Items.Add('Please enter the path of your Statics0.mul file in');
        MainListBox.Items.Add('the middle box. It will turn green when WorldForge');
        MainListBox.Items.Add('found the file.');
      end;
      WSCName:
      begin
        MainListBox.Items.Add('Please enter the path of the file you got from');
        MainListBox.Items.Add('the server operator in the right box. It will');
        MainListBox.Items.Add('turn green when WorldForge found the file.');
      end;
      GetIndex:
      begin
        MainListBox.Items.Add('Click on the "GetStaticIndex"-button to the right.');
        MainListBox.Items.Add('This can take up to 30 seconds! Please wait until');
        MainListBox.Items.Add('WorldForge has finished.');
      end;
      FreezeNow:
      begin
        MainListBox.Items.Add('Click on the "Freeze"-button to the right.');
        MainListBox.Items.Add('This can take up to some minutes! Please wait until');
        MainListBox.Items.Add('WorldForge has finished.');
      end;
      Finished:
      begin
        MainListBox.Items.Add('Finished! You can now close WorldForge and connect to');
        MainListBox.Items.Add('the server. Enjoy!');
      end;
    end;}
end; *)

// Events
// 2004/05/25 - Shinigami
Procedure TWFDiffForm.FormShow(Sender: TObject);
Var MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
    StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName: String;
Begin
  Assert(SizeOf(TWFDInfo)=1024, 'SizeOf(TWFDInfo) is not 1024, do not use WF-Diff!');

  LoadFileNames(MapFileName, StaIdxFileName, StaticsFileName, MapDifFileName, MapDiflFileName,
    StaDifFileName, StaDifiFileName, StaDiflFileName, OMapFileName, OStaIdxFileName, OStaticsFileName);
  MMapEdit.Text:=MapFileName;MStaIdxEdit.Text:=StaIdxFileName;MStaticsEdit.Text:=StaticsFileName;
  MapDifEdit.Text:=MapDifFileName;MapDiflEdit.Text:=MapDiflFileName;
  StaDifEdit.Text:=StaDifFileName;StaDifiEdit.Text:=StaDifiFileName;StaDiflEdit.Text:=StaDiflFileName;
  OMapEdit.Text:=OMapFileName;OStaIdxEdit.Text:=OStaIdxFileName;OStaticsEdit.Text:=OStaticsFileName;

  If MainForm.MapComboBox.ItemIndex<0 Then Caption:='WF-Diff - undefiniert' Else Begin
    Caption:='WF-Diff - '+MainForm.MapComboBox.Items[MainForm.MapComboBox.ItemIndex];

    Case MainForm.MapComboBox.ItemIndex of
      Map0_Britannia: Begin
                        MapCheckBox.Caption:='Map0.mul';StaticsCheckBox.Caption:='StaIdx0.mul + Statics0.mul';
                        MapLabel.Caption:='Map0.mul';StaIdxLabel.Caption:='StaIdx0.mul';StaticsLabel.Caption:='Statics0.mul';
                      End;
      Map1_Britannia: Begin
                        MapCheckBox.Caption:='Map0.mul';StaticsCheckBox.Caption:='StaIdx0.mul + Statics0.mul';
                        MapLabel.Caption:='Map0.mul';StaIdxLabel.Caption:='StaIdx0.mul';StaticsLabel.Caption:='Statics0.mul';
                      End;
      Map2_Illshenar: Begin
                        MapCheckBox.Caption:='Map2.mul';StaticsCheckBox.Caption:='StaIdx2.mul + Statics2.mul';
                        MapLabel.Caption:='Map2.mul';StaIdxLabel.Caption:='StaIdx2.mul';StaticsLabel.Caption:='Statics2.mul';
                      End;
      Map3_Malas:     Begin
                        MapCheckBox.Caption:='Map3.mul';StaticsCheckBox.Caption:='StaIdx3.mul + Statics3.mul';
                        MapLabel.Caption:='Map3.mul';StaIdxLabel.Caption:='StaIdx3.mul';StaticsLabel.Caption:='Statics3.mul';
                      End;
      Map4_Tokuno:    Begin
                        MapCheckBox.Caption:='Map4.mul';StaticsCheckBox.Caption:='StaIdx4.mul + Statics4.mul';
                        MapLabel.Caption:='Map4.mul';StaIdxLabel.Caption:='StaIdx4.mul';StaticsLabel.Caption:='Statics4.mul';
                      End;
    End;
  End;
End;

procedure TWFDiffForm.MapCheckBoxClick(Sender: TObject);
begin
  if (not MapCheckBox.Checked) and (not StaticsCheckBox.Checked) then
    StaticsCheckBox.Checked := True;
end;

procedure TWFDiffForm.StaticsCheckBoxClick(Sender: TObject);
begin
  if (not MapCheckBox.Checked) and (not StaticsCheckBox.Checked) then
    MapCheckBox.Checked := True;
end;

// 2004/05/26 - Shinigami
Procedure TWFDiffForm.OSIDiffCheckBoxClick(Sender: TObject);
Begin
  If OSIDiffCheckBox.Checked Then
    GenerateButton.Caption:='Generate OSI-Diff' Else GenerateButton.Caption:='Generate WF-Diff';
End;

// 2004/05/26 - Shinigami
Procedure TWFDiffForm.FilenameChange(Sender: TObject);
Begin
  If FileExists(TEdit(Sender).Text) Then TEdit(Sender).Color:=clGreen Else TEdit(Sender).Color:=clRed;
  UpdateEnabled;
End;

// 2004/05/06 - Birdy
// 2004/05/26 - Shinigami
procedure TWFDiffForm.GenerateButtonClick(Sender: TObject);
begin
  MainForm.lblStatus.Color := clRed;
  MainForm.lblStatus.Caption := 'BUSY';
  {StaticsCheckBox.Enabled := False;
  MapCheckBox.Enabled := False;
  GenerateButton.Enabled := False;
  ApplyButton.Enabled := False;
  OStaIdxEdit.Enabled := False;
  OStaticsEdit.Enabled := False;
  OMapEdit.Enabled := False;
  MMapEdit.Enabled := False;}
  WFDiffForm.Enabled := False;

  If Not OSIDiffCheckBox.Checked Then Begin
    If WFDFileSaveDialog.Execute Then GenerateDiff(OStaIdxEdit.Text, OStaticsEdit.Text, OMapEdit.Text,
      MStaIdxEdit.Text, MStaticsEdit.Text, MMapEdit.Text, WFDFileSaveDialog.FileName, SysProgress);
  End Else GenerateOSIDiff(OStaIdxEdit.Text, OStaticsEdit.Text, OMapEdit.Text,
    MStaIdxEdit.Text, MStaticsEdit.Text, MMapEdit.Text, SysProgress);

  WFDiffForm.Enabled := True;
  UpdateEnabled;
  MainForm.lblStatus.Color := clGreen;
  MainForm.lblStatus.Caption := 'READY';
end;

procedure TWFDiffForm.ApplyButtonClick(Sender: TObject);
var
  Ext: string;
begin
  {StaticsCheckBox.Enabled := False;
  MapCheckBox.Enabled := False;
  GenerateButton.Enabled := False;
  ApplyButton.Enabled := False;
  OStaIdxEdit.Enabled := False;
  OStaticsEdit.Enabled := False;
  OMapEdit.Enabled := False;
  MMapEdit.Enabled := False;}
  WFDiffForm.Enabled := False;
  if WFDFileOpenDialog.Execute then
  begin
    Ext := ExtractFileExt(WFDFileOpenDialog.FileName);
    if LowerCase(Ext) = '.wf2' then
      ApplyDiff(OStaIdxEdit.Text, OStaticsEdit.Text, OMapEdit.Text,
                MStaIdxEdit.Text, MStaticsEdit.Text, MMapEdit.Text,
                WFDFileOpenDialog.FileName,
                SysProgress)
    else
      MessageDlg('You can''t apply this file! If you want to apply a .wfd file, please use WorldForge 5.', mtError, [mbCancel], 0);
  end;
  WFDiffForm.Enabled := True;
  UpdateEnabled;
end;

procedure TWFDiffForm.CloseBtnClick(Sender: TObject);
begin
	Close;
end;

end.
