// Copyright (c) 1998, 1999, 2000 Jan Luebbe. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Jan Luebbe can be emailed at jluebbe@lasnet.de

unit WFFillClear;

interface

uses
  Windows, Messages, Classes, Controls, Forms, Dialogs, Main, StdCtrls,
  ExtCtrls, ComCtrls, Buttons;

type
  TWFFillClearForm = class(TForm)
    FileNamePanel: TPanel;
    StaIdxLabel: TLabel;
    StaticsLabel: TLabel;
    MapLabel: TLabel;
    MStaIdxEdit: TEdit;
    MStaticsEdit: TEdit;
    MMapEdit: TEdit;
    TopPanel: TPanel;
    ClearSelectGroupBox: TGroupBox;
    MapCheckBox: TCheckBox;
    StaticsCheckBox: TCheckBox;
    GoButton: TButton;
    FillClearAreaGroupBox: TGroupBox;
    ClearAreaTopEdit: TEdit;
    ClearAreaRightEdit: TEdit;
    ClearAreaLeftEdit: TEdit;
    ClearAreaBottomEdit: TEdit;
    ClearAreaTopLabel: TLabel;
    ClearAreaBottomLabel: TLabel;
    ClearAreaRightLabel: TLabel;
    ClearAreaLeftLabel: TLabel;
    ClearAreaNoteLabel: TLabel;
    ReplaceGroupBox: TGroupBox;
    StaticsClearInfoLabel: TLabel;
    FillTileLabel: TLabel;
    FillTileComboBox: TComboBox;
    FillAltitudeLabel: TLabel;
    FillAltitudeEdit: TEdit;
    FillAltitudeUpDown: TUpDown;
    BitBtn1: TBitBtn;
    procedure MStaIdxEditChange(Sender: TObject);
    procedure MStaticsEditChange(Sender: TObject);
    procedure MMapEditChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MapCheckBoxClick(Sender: TObject);
    procedure StaticsCheckBoxClick(Sender: TObject);
    procedure FillAltitudeEditExit(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    // UI
    procedure UpdateEnabled;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  WFFillClearForm: TWFFillClearForm;

// Registry
procedure LoadFileNames(var MStaIdxFileName, MStaticsFileName, MMapFileName: string);
procedure SaveFileNames(const MStaIdxFileName, MStaticsFileName, MMapFileName: string);

implementation

{$R *.DFM}

uses SysUtils, Graphics, Registry, Tools, WFTools, Progress;

// Registry
procedure LoadFileNames(var MStaIdxFileName, MStaticsFileName, MMapFileName: string);
var
  ARegistry: TRegistry;
begin
  ARegistry := TRegistry.Create;
  ARegistry.OpenKey('\Software\Ridcully\WorldForge', False);
  MStaIdxFileName := ARegistry.ReadString('StaIdx0.mul');
  MStaticsFileName := ARegistry.ReadString('Statics0.mul');
  MMapFileName := ARegistry.ReadString('Map0.mul');
  ARegistry.CloseKey;
  ARegistry.Free;
end;

procedure SaveFileNames(const MStaIdxFileName, MStaticsFileName, MMapFileName: string);
var
  ARegistry: TRegistry;
begin
  ARegistry := TRegistry.Create;
  ARegistry.OpenKey('\Software\Ridcully\WorldForge', True);
  ARegistry.WriteString('StaIdx0.mul', MStaIdxFileName);
  ARegistry.WriteString('Statics0.mul', MStaticsFileName);
  ARegistry.WriteString('Map0.mul', MMapFileName);
  ARegistry.CloseKey;
  ARegistry.Free;
end;

// UI
procedure TWFFillClearForm.UpdateEnabled;
begin
  with WFFillClearForm do
  begin
    if (MStaIdxEdit.Color = clGreen) and
       (MStaticsEdit.Color = clGreen) and
       (MMapEdit.Color = clGreen) then
    begin
      SaveFileNames(MStaIdxEdit.Text, MStaticsEdit.Text, MMapEdit.Text);
      GoButton.Enabled := True;
    end
    else
    begin
      GoButton.Enabled := False;
    end;
  end;
end;

procedure TWFFillClearForm.MStaIdxEditChange(Sender: TObject);
begin
  if FileExists(MStaIdxEdit.Text) and (LowerCase(ExtractFileName(MStaIdxEdit.Text)) = 'staidx0.mul')
    then MStaIdxEdit.Color := clGreen
    else MStaIdxEdit.Color := clRed;
  UpdateEnabled;
end;

procedure TWFFillClearForm.MStaticsEditChange(Sender: TObject);
begin
  if FileExists(MStaticsEdit.Text) and (LowerCase(ExtractFileName(MStaticsEdit.Text)) = 'statics0.mul')
    then MStaticsEdit.Color := clGreen
    else MStaticsEdit.Color := clRed;
  UpdateEnabled;
end;

procedure TWFFillClearForm.MMapEditChange(Sender: TObject);
begin
  if FileExists(MMapEdit.Text) and (LowerCase(ExtractFileName(MMapEdit.Text)) = 'map0.mul')
    then MMapEdit.Color := clGreen
    else MMapEdit.Color := clRed;
  UpdateEnabled;
end;

procedure TWFFillClearForm.FormShow(Sender: TObject);
var
  MStaIdxFileName, MStaticsFileName, MMapFileName: string;
begin
  LoadFileNames(MStaIdxFileName, MStaticsFileName, MMapFileName);
  MStaIdxEdit.Text := MStaIdxFileName;
  MStaticsEdit.Text := MStaticsFileName;
  MMapEdit.Text := MMapFileName;
end;

procedure TWFFillClearForm.MapCheckBoxClick(Sender: TObject);
begin
  if (not MapCheckBox.Checked) and (not StaticsCheckBox.Checked) then
    StaticsCheckBox.Checked := True;
  StaticsClearInfoLabel.Enabled := StaticsCheckBox.Checked;
  FillTileLabel.Enabled := MapCheckBox.Checked;
  FillTileComboBox.Enabled := MapCheckBox.Checked;
  FillAltitudeLabel.Enabled := MapCheckBox.Checked;
  FillAltitudeEdit.Enabled := MapCheckBox.Checked;
end;

procedure TWFFillClearForm.StaticsCheckBoxClick(Sender: TObject);
begin
  if (not MapCheckBox.Checked) and (not StaticsCheckBox.Checked) then
    MapCheckBox.Checked := True;
  StaticsClearInfoLabel.Enabled := StaticsCheckBox.Checked;
  FillTileLabel.Enabled := MapCheckBox.Checked;
  FillTileComboBox.Enabled := MapCheckBox.Checked;
  FillAltitudeLabel.Enabled := MapCheckBox.Checked;
  FillAltitudeEdit.Enabled := MapCheckBox.Checked;
end;

procedure TWFFillClearForm.FillAltitudeEditExit(Sender: TObject);
var
  temp: integer;
begin
  temp := StrToInt(FillAltitudeEdit.Text);
  if temp > FillAltitudeUpDown.Max then FillAltitudeEdit.Text := IntToStr(FillAltitudeUpDown.Max);
  if temp < FillAltitudeUpDown.Min then FillAltitudeEdit.Text := IntToStr(FillAltitudeUpDown.Min);
end;

procedure TWFFillClearForm.GoButtonClick(Sender: TObject);
var
  CurrentFile: file of byte;
  X, Y, CX, CY, i: word;
  Top, Left, Right, Bottom: word;
  StaticPointer: TStaticPointer;
  MapBlock: TMapBlock;
  StaticBlock: TStaticBlock;
  NamedTileIndex: integer;
begin
  ProgressForm.Show;
  MainForm.lblStatus.Color := clRed;
  MainForm.lblStatus.Caption := 'BUSY';
  ZeroMemory(Addr(StaticPointer), SizeOf(TStaticPointer));
  ZeroMemory(Addr(MapBlock), SizeOf(TMapBlock));
  begin
    Top := StrToInt(ClearAreaTopEdit.Text);
    Left := StrToInt(ClearAreaLeftEdit.Text);
    Right := StrToInt(ClearAreaRightEdit.Text);
    Bottom := StrToInt(ClearAreaBottomEdit.Text);
  end;
  if StaticsCheckBox.Checked then
  begin
    // StaIdx?.mul
    StaticPointer.Start := $FFFFFFFF;
    StaticPointer.Length := $FFFFFFFF;
    StaticPointer.Unknown := 0;
    AssignFile(CurrentFile, MStaIdxEdit.Text);
    Reset(CurrentFile);
    Seek(CurrentFile, 0);
    for X := 0 to MainForm.MapWidth-1 do
    begin
      for Y := 0 to MainForm.MapHeight-1 do
        case CalcBlockCoverage(Top, Left, Right, Bottom, X, Y) of
          PartlyInArea:
          begin
            // read old StaticBlock
            Seek(CurrentFile, (X*MainForm.MapHeight+Y)*SizeOf(TStaticPointer));
            BlockRead(CurrentFile, StaticPointer, SizeOf(TStaticPointer));
            GetStaticBlock(MStaticsEdit.Text, StaticPointer, StaticBlock);
            // clear part of the block
            i := 1;
            while i <= StaticBlock.Count do
            begin
              CX := X*8+StaticBlock.Rec[i].X;
              CY := Y*8+StaticBlock.Rec[i].Y;
              if (Top <= CY) and
                (Left <= CX) and
                (Right >= CX) and
                (Bottom >= CY) then // inside clear area
              begin
                StaticBlock.Rec[i] := StaticBlock.Rec[StaticBlock.Count];
                Dec(StaticBlock.Count);
              end
              else
                Inc(i);
            end;
            // write new StaticBlock
            PutStaticBlock(MStaticsEdit.Text, StaticPointer, StaticBlock);
            Seek(CurrentFile, (X*MainForm.MapHeight+Y)*SizeOf(TStaticPointer));
            BlockWrite(CurrentFile, StaticPointer, SizeOf(TStaticPointer));
          end;
          CompletelyInArea:
          begin
            StaticPointer.Start := $FFFFFFFF;
            StaticPointer.Length := $FFFFFFFF;
            StaticPointer.Unknown := 0;
            Seek(CurrentFile, (X*MainForm.MapHeight+Y)*SizeOf(TStaticPointer));
            BlockWrite(CurrentFile, StaticPointer, SizeOf(TStaticPointer));
          end;
        end;
      SysProgress(X*100 div (MainForm.MapWidth-1));
      Application.ProcessMessages;
    end;
    CloseFile(CurrentFile);
  end;
  if MapCheckBox.Checked then
  begin
    // Map?.mul
    NamedTileIndex := GetNamedTileIndex(FillTileComboBox.Text);
    AssignFile(CurrentFile, MMapEdit.Text);
    Reset(CurrentFile);
    Seek(CurrentFile, 0);
    for X := 0 to MainForm.MapWidth-1 do
    begin
      for Y := 0 to MainForm.MapHeight-1 do
      begin
        case CalcBlockCoverage(Top, Left, Right, Bottom, X, Y) of // in the block contains cells to be preserved, read it
          OutOfArea: continue; // do not write or read anything, next block
          PartlyInArea:
          begin
            Seek(CurrentFile, (X*MainForm.MapHeight+Y)*SizeOf(TMapBlock));
            BlockRead(CurrentFile, MapBlock, SizeOf(TMapBlock));
          end;
        end;

        for CX := 0 to 7 do
          for CY := 0 to 7 do
            if (Top <= Y*8+CY) and
              (Left <= X*8+CX) and
              (Right >= X*8+CX) and
              (Bottom >= Y*8+CY) then // inside clear area
            begin
              MapBlock.Cells[CY, CX].Altitude := StrToInt(FillAltitudeEdit.Text);
              if NamedTileIndex >= 0 then
                MapBlock.Cells[CY, CX].TileID := GetNamedTile(NamedTileIndex)
              else
                MapBlock.Cells[CY, CX].TileID := SpacedHexToInt(FillTileComboBox.Text, 2);
            end;
        Seek(CurrentFile, (X*MainForm.MapHeight+Y)*SizeOf(TMapBlock));
        BlockWrite(CurrentFile, MapBlock, SizeOf(TMapBlock));
      end;
      SysProgress(X*100 div (MainForm.MapWidth-1));
      Application.ProcessMessages;
    end;
    CloseFile(CurrentFile);
  end;
  MainForm.lblStatus.Color := clGreen;
  MainForm.lblStatus.Caption := 'Ready';
  ProgressForm.Hide;
end;

procedure TWFFillClearForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  ClearAreaTopEdit.Text := '0';
  ClearAreaLeftEdit.Text := '0';
  ClearAreaRightEdit.Text := IntToStr(MainForm.MapWidth*8-1);
  ClearAreaBottomEdit.Text := IntToStr(MainForm.MapHeight*8-1);
  for i := 0 to NamedTileCount-1 do
  begin
    FillTileComboBox.Items.Add(NamedTiles[i].Name);
  end;
end;

procedure TWFFillClearForm.BitBtn1Click(Sender: TObject);
begin
	Close;
end;

end.
