// Copyright (c) 1998, 1999, 2000 Jan Luebbe. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Jan Luebbe can be emailed at jluebbe@lasnet.de

unit WFPMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls;

type
  TWFPMainForm = class(TForm)
    PageControl: TPageControl;
    NextButton: TButton;
    _ProgressBar: TProgressBar;
    FinishButton: TButton;
    ActionLabel: TLabel;
    CheckHDTabSheet: TTabSheet;
    CheckCDTabSheet: TTabSheet;
    ApplyTabSheet: TTabSheet;
    UOClientPathLabel: TLabel;
    UOClientPathEdit: TEdit;
    UOClientResultsGroupBox: TGroupBox;
    UOClientStaIdxNameLabel: TLabel;
    UOClientStaIdxResultLabel: TLabel;
    UOClientStaticsNameLabel: TLabel;
    UOClientStaticsResultLabel: TLabel;
    UOClientMapNameLabel: TLabel;
    UOClientMapResultLabel: TLabel;
    UOCDPathEdit: TEdit;
    UOCDStaIdxResultLabel: TLabel;
    UOCDStaticsResultLabel: TLabel;
    UOCDMapResultLabel: TLabel;
    UOCDResultsGroupBox: TGroupBox;
    UOCDMapNameLabel: TLabel;
    UOCDStaticsNameLabel: TLabel;
    UOCDStaIdxNameLabel: TLabel;
    UOCDPathLabel: TLabel;
    UOCDTypeResultLabel: TLabel;
    UOCDTypeNameLabel: TLabel;
    UOClientTestButton: TButton;
    UOCDRecheckButton: TButton;
    procedure UOClientPathEditChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure FinishButtonClick(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure UOCDPathEditChange(Sender: TObject);
    procedure UOClientTestButtonClick(Sender: TObject);
    procedure UOCDRecheckButtonClick(Sender: TObject);
  private
    // UI
    procedure UpdateEnabled;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  WFPMainForm: TWFPMainForm;

// File tools
function FileWriteable(const FileName: string):boolean;

procedure _SetProgressBar(Position: integer);

implementation

{$R *.DFM}

uses
  WFDiff;

var
  WFDFileName: string;
  UOClientPathOK: boolean = False;
  UOCDPathOK: boolean = False;

// File tools
function FileWriteable(const FileName: string):boolean;
begin
  if not (FileGetAttr(FileName)=-1) then // Are the Attrs readable?
    if (FileGetAttr(FileName) and faReadOnly) = 0 then // Is file writeable?
      Result := True
    else
      Result := False
  else
    Result := False; // Error
end;

// UI
procedure TWFPMainForm.UpdateEnabled;
begin
  ActionLabel.Caption := PageControl.ActivePage.Caption;
  if PageControl.ActivePage = PageControl.Pages[0] then
  begin
    if UOClientPathOK then
      begin
        NextButton.Enabled := True;
        FinishButton.Enabled := False;
      end
    else
      begin
        NextButton.Enabled := False;
        FinishButton.Enabled := False;
      end;
  end
  else if PageControl.ActivePage = PageControl.Pages[1] then
  begin
    if UOCDPathOK then
      begin
        NextButton.Enabled := True;
        FinishButton.Enabled := False;
      end
    else
      begin
        NextButton.Enabled := False;
        FinishButton.Enabled := False;
      end;
  end
  else if PageControl.ActivePage = PageControl.Pages[2] then
  begin
      begin
        NextButton.Enabled := False;
        FinishButton.Enabled := False;
      end;
  end
  else // shouldn't happen!
  begin
    NextButton.Enabled := False;
    FinishButton.Enabled := False;
  end;
end;

procedure _SetProgressBar(Position: integer);
begin
  WFPMainForm._ProgressBar.Position := Position;
end;

procedure TWFPMainForm.UOClientPathEditChange(Sender: TObject);
var
  UOClientPath: string;
begin
  UOClientPath := UOClientPathEdit.Text;
  while (Length(UOClientPath) > 0) and (UOClientPath[Length(UOClientPath)]='\') do // Workaround for C:\\ problem
  begin
    UOClientPath[Length(UOClientPath)] := ' ';
    UOClientPath := Trim(UOClientPath);
  end;
  UOClientPath := Trim(UOClientPath);
  UOClientPathOK := True;

  if FileExists(UOClientPath+'\StaIdx0.mul') then
    if FileWriteable(UOClientPath+'\StaIdx0.mul') then
      begin
        UOClientStaIdxResultLabel.Caption := 'Found and writeable!';
        UOClientStaIdxResultLabel.Color := clGreen;
      end
    else
    begin
      UOClientPathOK := False;
      UOClientStaIdxResultLabel.Caption := 'Found, but readonly!';
      UOClientStaIdxResultLabel.Color := clRed;
    end
  else
  begin
    UOClientPathOK := False;
    UOClientStaIdxResultLabel.Caption := 'Not found!';
    UOClientStaIdxResultLabel.Color := clRed;
  end;

  if FileExists(UOClientPath+'\Statics0.mul') then
    if FileWriteable(UOClientPath+'\Statics0.mul') then
      begin
        UOClientStaticsResultLabel.Caption := 'Found and writeable!';
        UOClientStaticsResultLabel.Color := clGreen;
      end
    else
    begin
      UOClientPathOK := False;
      UOClientStaticsResultLabel.Caption := 'Found, but readonly!';
      UOClientStaticsResultLabel.Color := clRed;
    end
  else
  begin
    UOClientPathOK := False;
    UOClientStaticsResultLabel.Caption := 'Not found!';
    UOClientStaticsResultLabel.Color := clRed;
  end;

  if FileExists(UOClientPath+'\Map0.mul') then
    if FileWriteable(UOClientPath+'\Map0.mul') then
      begin
        UOClientMapResultLabel.Caption := 'Found and writeable!';
        UOClientMapResultLabel.Color := clGreen;
      end
    else
    begin
      UOClientPathOK := False;
      UOClientMapResultLabel.Caption := 'Found, but readonly!';
      UOClientMapResultLabel.Color := clRed;
    end
  else
  begin
    UOClientPathOK := False;
    UOClientMapResultLabel.Caption := 'Not found!';
    UOClientMapResultLabel.Color := clRed;
  end;

  UpdateEnabled;
end;

procedure TWFPMainForm.FormShow(Sender: TObject);
begin
  if (ParamCount < 1) or (ParamCount = 2) or (ParamCount > 3) then
  begin
    Application.MessageBox('Usage: wfpatch.exe wfdfile [UOClientPath UOCDPath]', 'Error', 0);
    Application.Terminate;
  end;
  if ParamCount = 1 then
  begin
    WFDFileName := ParamStr(1);
    UOClientPathEdit.Text := '';
    UOCDPathEdit.Text := '';
  end;
  if ParamCount = 3 then
  begin
    WFDFileName := ParamStr(1);
    UOClientPathEdit.Text := ParamStr(2);
    UOCDPathEdit.Text := ParamStr(3);
  end;

  if not FileExists(WFDFileName) then
  begin
    Application.MessageBox(PChar(WFDFileName+' does not exist!'), 'Error', 0);
    Application.Terminate;
  end;

  if not (LowerCase(ExtractFileExt(WFDFileName))='.wf2') then
  begin
    Application.MessageBox(PChar(WFDFileName+' is not a .wf2 file!'), 'Error', 0);
    Application.Terminate;
  end;

  PageControl.ActivePage := PageControl.Pages[0];
  UpdateEnabled;

  //WFDiff._SetProgressBar := WFPMain._SetProgressBar;
end;

procedure TWFPMainForm.NextButtonClick(Sender: TObject);
begin
  //PageControl.SelectNextPage(True);
  if PageControl.ActivePage = PageControl.Pages[0] then
    PageControl.ActivePage := PageControl.Pages[1]
  else if PageControl.ActivePage = PageControl.Pages[1] then
    PageControl.ActivePage := PageControl.Pages[2];
  UpdateEnabled;
end;

procedure TWFPMainForm.FinishButtonClick(Sender: TObject);
begin
  // Apply patch
end;

procedure TWFPMainForm.PageControlChange(Sender: TObject);
begin
  UpdateEnabled;
end;

procedure TWFPMainForm.UOCDPathEditChange(Sender: TObject);
var
  UOCDPath: string;
  StaticsFile: file of byte;
  StaticsFileSize: integer;
begin
  UOCDPath := UOCDPathEdit.Text;
  while (Length(UOCDPath) > 0) and (UOCDPath[Length(UOCDPath)]='\') do // Workaround for C:\\ problem
  begin
    UOCDPath[Length(UOCDPath)] := ' ';
    UOCDPath := Trim(UOCDPath);
  end;
  UOCDPath := Trim(UOCDPath);
  UOCDPathOK := True;

  if FileExists(UOCDPath+'\StaIdx0.mul') then
    begin
      UOCDStaIdxResultLabel.Caption := 'Found!';
      UOCDStaIdxResultLabel.Color := clGreen;
    end
  else
  begin
    UOCDPathOK := False;
    UOCDStaIdxResultLabel.Caption := 'Not found!';
    UOCDStaIdxResultLabel.Color := clRed;
  end;

  if FileExists(UOCDPath+'\Statics0.mul') then
    begin
      UOCDStaticsResultLabel.Caption := 'Found!';
      UOCDStaticsResultLabel.Color := clGreen;
    end
  else
  begin
    UOCDPathOK := False;
    UOCDStaticsResultLabel.Caption := 'Not found!';
    UOCDStaticsResultLabel.Color := clRed;
  end;

  if FileExists(UOCDPath+'\Map0.mul') then
    begin
      UOCDMapResultLabel.Caption := 'Found!';
      UOCDMapResultLabel.Color := clGreen;
    end
  else
  begin
    UOCDPathOK := False;
    UOCDMapResultLabel.Caption := 'Not found!';
    UOCDMapResultLabel.Color := clRed;
  end;

  if UOCDPathOK then
  begin
    AssignFile(StaticsFile, UOCDPath+'\Statics0.mul');
    Reset(StaticsFile);
    StaticsFileSize := FileSize(StaticsFile);
    CloseFile(StaticsFile);
    if StaticsFileSize = 21971754 then
      UOCDTypeResultLabel.Caption := 'The Second Age'
    else if StaticsFileSize = 19171873 then
      UOCDTypeResultLabel.Caption := 'Original (not T2A)'
    else
      UOCDTypeResultLabel.Caption := 'Unknown'
  end
  else
    UOCDTypeResultLabel.Caption := 'Path not correct!';

  UpdateEnabled;
end;

procedure TWFPMainForm.UOClientTestButtonClick(Sender: TObject);
begin
  UOClientPathEdit.OnChange(Sender);
  UpdateEnabled;
end;

procedure TWFPMainForm.UOCDRecheckButtonClick(Sender: TObject);
begin
  UOCDPathEdit.OnChange(Sender);
  UpdateEnabled;
end;

end.
