// Copyright (c) 1998, 1999, 2000 Jan Luebbe. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Jan Luebbe can be emailed at jluebbe@lasnet.de

unit WFTools;

interface

uses SysUtils, Classes;

type
  TNamedTile = record
    Name: string;
    StartTileID: word;
    TileIDCount: integer;
  end;

  TBookCoord = record
    Col : Integer;
    Row : Integer;
    isset : Boolean;
  end;

  TrBrush = record
    Col : integer;
    Row : integer;
    id : integer;
  end;
  
TTileNamed = class(TObject)
  private
  FTileName : String;
  FTileId : TStringList;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    function getRandomTileID : Integer;
  published
    property TileName : String read FTileName write FTileName;
    property TileIds : TStringList read FTileId write FTileId;
end;

var
  iNamedTileCount : Integer = 43;
  BookMarks : array[0..9] of TBookCoord;

const
  NamedTileCount = 43;
  NamedTiles: array[0..NamedTileCount-1] of TNamedTile =
    (
       (Name: 'Random cave'; StartTileID: $0245; TileIDCount: 5),
       (Name: 'Random cobblestones'; StartTileID: $03E9; TileIDCount: 4),
       (Name: 'Random desert'; StartTileID: $0016; TileIDCount: 4),
       (Name: 'Random dirt (big stones)'; StartTileID: $0071; TileIDCount: 4),
       (Name: 'Random dirt (small stones)'; StartTileID: $0075; TileIDCount: 4),
       (Name: 'Random dirt (all stones)'; StartTileID: $0071; TileIDCount: 8),
       (Name: 'Random forest'; StartTileID: $00C4; TileIDCount: 4),
       (Name: 'Random grass'; StartTileID: $0003; TileIDCount: 4),
       (Name: 'Random jungle'; StartTileID: $00AC; TileIDCount: 4),
       (Name: 'Random lava'; StartTileID: $01F4; TileIDCount: 4),
       (Name: 'Random mountain'; StartTileID: $022C; TileIDCount: 4),
       (Name: 'Random snow'; StartTileID: $011A; TileIDCount: 4),
       (Name: 'Random stars'; StartTileID: $01FA; TileIDCount: 6),
       (Name: 'Random water'; StartTileID: $00A8; TileIDCount: 4),
       (Name: 'Random brick (Brown E/W)'; StartTileID: $048F; TileIDCount: 4),
       (Name: 'Random brick (Brown N/S)'; StartTileID: $0493; TileIDCount: 4),
       (Name: 'Random brick (Light Sand E/W)'; StartTileID: $0501; TileIDCount: 2),
       (Name: 'Random brick (Light Sand N/S)'; StartTileID: $0503; TileIDCount: 2),
       (Name: 'Random brick (Dark Sand E/W)'; StartTileID: $050D; TileIDCount: 2),
       (Name: 'Random brick (Dark Sand N/S)'; StartTileID: $050F; TileIDCount: 2),
       (Name: 'Random brick (Red E/W)'; StartTileID: $047D; TileIDCount: 2),
       (Name: 'Random brick (Red N/S)'; StartTileID: $0480; TileIDCount: 4),
       (Name: 'Random dark wooden floor (E/W)'; StartTileID: $0406; TileIDCount: 4),
       (Name: 'Random dark wooden floor (N/S)'; StartTileID: $040A; TileIDCount: 4),
       (Name: 'Random flagstone (Sand)'; StartTileID: $0201; TileIDCount: 4),
       (Name: 'Random flagstone (Stone)'; StartTileID: $04EB; TileIDCount: 4),
       (Name: 'Random light wooden floor (E/W)'; StartTileID: $040E; TileIDCount: 4),
       (Name: 'Random light wooden floor (N/S)'; StartTileID: $0412; TileIDCount: 4),
       (Name: 'Random marble (Green)'; StartTileID: $04E7; TileIDCount: 2),
       (Name: 'Random marble (Silver)'; StartTileID: $0486; TileIDCount: 4),
       (Name: 'Random marble (White Pattern)'; StartTileID: $04E9; TileIDCount: 2),
       (Name: 'Random marble (White)'; StartTileID: $04EF; TileIDCount: 4),
       (Name: 'Random sandstone (Light)'; StartTileID: $0446; TileIDCount: 8),
       (Name: 'Random sandstone (Dark)'; StartTileID: $044E; TileIDCount: 8),
       (Name: 'Random stone (Light)'; StartTileID: $0436; TileIDCount: 4),
       (Name: 'Random stone (Medium)'; StartTileID: $043A; TileIDCount: 4),
       (Name: 'Random stone (Dark)'; StartTileID: $043E; TileIDCount: 4),
       (Name: 'Random stone (Light Sand)'; StartTileID: $0442; TileIDCount: 2),
       (Name: 'Random stone (Dark Sand)'; StartTileID: $0444; TileIDCount: 2),
       (Name: 'Black Weird'; StartTileID: $3FCC; TileIDCount: 4),
       (Name: 'White Weird'; StartTileID: $3FEC; TileIDCount: 4),
       (Name: 'Water Weird'; StartTileID: $3FF0; TileIDCount: 4),
       (Name: 'Orange Marble Weird'; StartTileID: $3FF7; TileIDCount: 4)
       //(Name: 'Random '; StartTileID: $; TileIDCount: ),
    );


function GetNamedTileIndex(Name: string):integer;
function GetNamedTile(Name: string):word; overload;
function GetNamedTile(Index: integer):word; overload;
function TextToStringList(sep: PChar; a : String): TStringList;
function TextToBrush(sep: PChar; a : String): TrBrush;
function StartStr(const sep: PChar; const s : String) : String;
function EndStr(const sep: PChar; const s : String) : String;

implementation

constructor TTileNamed.Create;
begin
  inherited Create;
  FTileId := TStringList.Create;
end;

destructor TTileNamed.Destroy;
begin
  FTileId.Free;
  inherited Destroy;
end; 

function TTileNamed.getRandomTileID : Integer;
var
  a : String;
begin
  a := FTileId[Random(FTileId.Count)];
  result := StrToInt(a);
end;

function GetNamedTileIndex(Name: string):integer;
var
  i: integer;
begin
  for i := 0 to NamedTileCount-1 do
  begin
    if UpperCase(Name) = UpperCase(NamedTiles[i].Name) then
    begin
      Result := i;
      Exit;
    end;
  end;
  Result := -1;
end;

function GetNamedTile(Name: string):word;
var
  Index: integer;
begin
  Index := GetNamedTileIndex(Name);
  if Index = -1 then
    raise Exception.Create('Invalid tile name: '+Name);
  Result := NamedTiles[Index].StartTileID + Random(NamedTiles[Index].TileIDCount);
end;

function GetNamedTile(Index: integer):word;
begin
  Result := NamedTiles[Index].StartTileID + Random(NamedTiles[Index].TileIDCount);
end;

function TextToStringList(sep: PChar; a : String): TStringList;
var
  tempStr : String;
begin
  tempStr := a;
  Result  := TStringList.Create;
  try
    while Pos(sep, a) > 0 do
    begin
      tempStr := StartStr(sep,a);
      Result.Add(tempStr);
      Delete(a, 1, Length(tempStr) + 1);
    end;
    Result.Add(TrimRight(a)); // letzte Eerweiterung in Liste Einfgen
  except
    Result.Free;
  end;
end;

function TextToBrush(sep: PChar; a : String): TrBrush;
var
  tempStr : String;
  c : integer;
begin
  c := 0;
  tempStr := a;
  try
    while Pos(sep, a) > 0 do
    begin
      tempStr := StartStr(sep,a);
      if c = 0 then
        Result.Col := StrToInt(tempStr)
      else
        Result.Row := StrToInt(tempStr);
      Delete(a, 1, Length(tempStr) + 1);
      inc(c);
    end;
    Result.id := StrToInt(TrimRight(a)); // letzte Eerweiterung in Liste Einfgen
  except
  end;
end;

function StartStr(const sep: PChar; const s : String) : String;
var
  s1 : String;
begin
  s1 := s;
  delete(S1, Pos(sep, s1),Length(s1));
  StartStr := s1;
end;

function EndStr(const sep: PChar; const s : String) : String;
var
  s1 : String;
begin
  s1 := s;
  while Pos(sep, s1) > 0 do
    Delete(s1, 1, 1);
  endStr:=S1;
end;


end.
